/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.rcp.application.utils;

import java.io.IOException;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IPerspectiveDescriptor;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PlatformUI;
import org.fortiss.af3.project.AF3ProjectActivator;
import org.fortiss.af3.project.example.IExample;
import org.fortiss.af3.project.model.FileProject;
import org.fortiss.af3.project.ui.perspective.ModelingPerspectiveFactory;
import org.fortiss.af3.project.ui.utils.NewProjectUtils;
import org.fortiss.af3.project.utils.ProjectUtils;
import org.fortiss.tooling.kernel.utils.LoggingUtils;

/**
 * Utility methods for dealing with the UI.
 * 
 * @author becker
 */
public class ApplicationUtils {

	/**
	 * Switch to the AF3 modeling perspective (e.g., if being in the Simulator perspective, creating
	 * a new project should result in the change to the AF3 modeling perspective).
	 */
	public static void switchToAF3Perspective() {

		IWorkbench workbench = PlatformUI.getWorkbench();
		for(IPerspectiveDescriptor pDesc : workbench.getPerspectiveRegistry().getPerspectives()) {
			if(pDesc.getLabel().equals(ModelingPerspectiveFactory.MODELING_PERSPECTIVE_NAME)) {
				if(workbench.getActiveWorkbenchWindow() != null &&
						workbench.getActiveWorkbenchWindow().getActivePage() != null) {
					workbench.getActiveWorkbenchWindow().getActivePage().setPerspective(pDesc);
				}
				break;
			}
		}
		refreshWorkspace();
	}

	/** Refresh the workspace. */
	public static void refreshWorkspace() {
		try {
			// refresh the workspace
			ResourcesPlugin.getWorkspace().getRoot().refreshLocal(IResource.DEPTH_INFINITE, null);
		} catch(Exception e) {
			LoggingUtils.error(AF3ProjectActivator.getDefault(), "Cannot refresh workspace!", e);
			e.printStackTrace();
			MessageDialog.openError(Display.getDefault().getActiveShell(), "Error",
					"Cannot refresh workspace!\n" + e.toString());
		}
	}

	/**
	 * Checks if a file with the given project name already exists in the
	 * GeneralProject-Directory. If not, an {@link IFile} with the same name is
	 * returned. If yes, the user is asked for another project name. After the
	 * user inserted a valid unused name, an {@link IFile} with this name is returned.
	 * 
	 * @param projectName
	 *            The default project name.
	 * @param suffix
	 *            The current AF3 project files extension.
	 * @return The {@link IFile} with this projectName and suffix.
	 */
	public static IFile validateProjectName(String projectName, String suffix) {
		// Get the general project that contains the AF3-Projects.
		final IResource resource = ResourcesPlugin.getWorkspace().getRoot()
				.findMember(NewProjectUtils.getDefaultGeneralProjectName());

		// If project name was given with file extension, remove this first.
		projectName = projectName.replace(suffix, "");

		// Create project file object.
		IFile projectFile = ((IContainer)resource).getFile(new Path(projectName + suffix));

		// If a file with this name already exists.
		if(projectFile.exists()) {
			String newProjectPath =
					NewProjectUtils.requestFileNameDialog(projectFile.getLocation().toFile());
			if(newProjectPath == null) {
				return null;
			}

			projectFile = ((IContainer)resource).getFile(new Path(newProjectPath));
		}

		return projectFile;
	}

	/**
	 * Creates a project (per code) and stores it.
	 * 
	 * @param projectName
	 *            the name of project to create
	 * @param suffix
	 *            the current AF3 project files extension
	 * @param builder
	 *            {@link IExample} builder
	 * @return whether the created project is saved to workspace
	 */
	public static boolean createAndSaveProject(String projectName, String suffix,
			IExample builder) {
		IFile projectFile = validateProjectName(projectName, suffix);
		if(projectFile != null) {
			// Save the project to workspace.
			return saveProject(builder.createProject(projectFile.getName().replace(suffix, "")),
					projectFile.getFullPath().toString());
		}
		return false;
	}

	/**
	 * Saves given project to given path.
	 * 
	 * @param project
	 *            the given project to save
	 * @param fullPath
	 *            to saving path
	 * @return whether the given project is saved to workspace
	 */
	public static boolean saveProject(FileProject project, String fullPath) {
		try {
			ProjectUtils.saveProject(project, fullPath);
			return true;
		} catch(final IOException e) {
			MessageDialog.openError(Display.getDefault().getActiveShell(), "Error", e.getMessage());
			return false;
		}
	}
}
